@@ -157,6 +157,60 @@ void ChartAxisElement::handleLabelsPositionChanged()
     presenter()->layout()->invalidate();
 }
 
+void ChartAxisElement::valueLabelEdited(qreal oldValue, qreal newValue)
+{
+    qreal range = max() - min();
+    qreal center = ((max() - min()) / 2.0) + min();
+    qreal newRange = 0.0;
+    auto label = static_cast<ValueAxisLabel *>(this->sender());
+    if ((oldValue >= center && newValue >= min())
+     || (oldValue <  center && newValue >= max() && oldValue != min())) {
+        newRange = range * ((newValue - min()) / (oldValue - min()));
+        if (newRange > 0) {
+            m_axis->setRange(min(), min() + newRange);
+            return;
+        }
+    } else if ((oldValue >= center && newValue <= min() && max() != oldValue)
+            || (oldValue <  center && newValue <  max())) {
+        newRange = range * ((max() - newValue) / (max() - oldValue));
+        if (newRange > 0) {
+            m_axis->setRange(max() - newRange, max());
+            return;
+        }
+    }
+    label->reloadBeforeEditContent();
+}
+
+void ChartAxisElement::dateTimeLabelEdited(const QDateTime &oldTime, const QDateTime &newTime)
+{
+    qreal range = max() - min();
+    qreal center = ((max() - min()) / 2.0) + min();
+    qreal newRange = 0.0;
+    qint64 oldValue = oldTime.toMSecsSinceEpoch();
+    qint64 newValue = newTime.toMSecsSinceEpoch();
+    if ((oldValue >= center && newValue >= min())
+     || (oldValue <  center && newValue >= max() && oldValue != min())) {
+        newRange = range * ((newValue - min()) / (oldValue - min()));
+        if (newRange > 0) {
+            m_axis->setRange(
+                        QDateTime::fromMSecsSinceEpoch(min()),
+                        QDateTime::fromMSecsSinceEpoch(min() + newRange));
+            return;
+        }
+    } else if ((oldValue >= center && newValue <= min() && max() != oldValue)
+            || (oldValue <  center && newValue <  max())) {
+        newRange = range * ((max() - newValue) / (max() - oldValue));
+        if (newRange > 0) {
+            m_axis->setRange(max() - newRange, max());
+            m_axis->setRange(
+                        QDateTime::fromMSecsSinceEpoch(max() - newRange),
+                        QDateTime::fromMSecsSinceEpoch(max()));
+            return;
+        }
+    }
+    static_cast<DateTimeAxisLabel *>(this->sender())->reloadBeforeEditContent();
+}
+
 void ChartAxisElement::handleLabelsVisibleChanged(bool visible)
 {
     QGraphicsLayoutItem::updateGeometry();
@@ -298,6 +352,24 @@ qreal ChartAxisElement::max() const
     return m_axis->d_ptr->max();
 }
 
+qreal ChartAxisElement::tickInterval() const
+{
+    QValueAxis *valueAxis = qobject_cast<QValueAxis *>(m_axis);
+    if (valueAxis)
+        return valueAxis->tickInterval();
+    else
+        return 0.0;
+}
+
+qreal ChartAxisElement::tickAnchor() const
+{
+    QValueAxis *valueAxis = qobject_cast<QValueAxis *>(m_axis);
+    if (valueAxis)
+        return valueAxis->tickAnchor();
+    else
+        return 0.0;
+}
+
 QString ChartAxisElement::formatLabel(const QString &formatSpec, const QByteArray &array,
                                       qreal value, int precision, const QString &preStr,
                                       const QString &postStr) const
@@ -310,13 +382,13 @@ QString ChartAxisElement::formatLabel(const QString &formatSpec, const QByteArra
             if (presenter()->localizeNumbers())
                 retVal = preStr + presenter()->locale().toString(qint64(value)) + postStr;
             else
-                retVal = QString().sprintf(array, qint64(value));
+                retVal = QString::asprintf(array.constData(), qint64(value));
         } else if (formatSpec.at(0) == QLatin1Char('u')
                  || formatSpec.at(0) == QLatin1Char('o')
                  || formatSpec.at(0) == QLatin1Char('x')
                  || formatSpec.at(0) == QLatin1Char('X')) {
             // These formats are not supported by localized numbers
-            retVal = QString().sprintf(array, quint64(value));
+            retVal = QString::asprintf(array.constData(), quint64(value));
         } else if (formatSpec.at(0) == QLatin1Char('f')
                    || formatSpec.at(0) == QLatin1Char('F')
                    || formatSpec.at(0) == QLatin1Char('e')
@@ -329,7 +401,7 @@ QString ChartAxisElement::formatLabel(const QString &formatSpec, const QByteArra
                                                          precision)
                         + postStr;
             } else {
-                retVal = QString().sprintf(array, value);
+                retVal = QString::asprintf(array.constData(), value);
             }
         }
     }
@@ -337,6 +409,8 @@ QString ChartAxisElement::formatLabel(const QString &formatSpec, const QByteArra
 }
 
 QStringList ChartAxisElement::createValueLabels(qreal min, qreal max, int ticks,
+                                                qreal tickInterval, qreal tickAnchor,
+                                                QValueAxis::TickType tickType,
                                                 const QString &format) const
 {
     QStringList labels;
@@ -346,9 +420,22 @@ QStringList ChartAxisElement::createValueLabels(qreal min, qreal max, int ticks,
 
     if (format.isEmpty()) {
         int n = qMax(int(-qFloor(std::log10((max - min) / (ticks - 1)))), 0) + 1;
-        for (int i = 0; i < ticks; i++) {
-            qreal value = min + (i * (max - min) / (ticks - 1));
-            labels << presenter()->numberToString(value, 'f', n);
+        if (tickType == QValueAxis::TicksFixed) {
+            for (int i = 0; i < ticks; i++) {
+                qreal value = min + (i * (max - min) / (ticks - 1));
+                labels << presenter()->numberToString(value, 'f', n);
+            }
+        } else {
+            qreal value = tickAnchor;
+            if (value > min)
+                value = value - int((value - min) / tickInterval) * tickInterval;
+            else
+                value = value + qCeil((min - value) / tickInterval) * tickInterval;
+
+            while (value <= max || qFuzzyCompare(value, max)) {
+                labels << presenter()->numberToString(value, 'f', n);
+                value += tickInterval;
+            }
         }
     } else {
         QByteArray array = format.toLatin1();
@@ -375,9 +462,22 @@ QStringList ChartAxisElement::createValueLabels(qreal min, qreal max, int ticks,
             if (format.indexOf(*labelFormatMatcher, 0, &rmatch) != -1)
                 formatSpec = rmatch.captured(1);
         }
-        for (int i = 0; i < ticks; i++) {
-            qreal value = min + (i * (max - min) / (ticks - 1));
-            labels << formatLabel(formatSpec, array, value, precision, preStr, postStr);
+        if (tickType == QValueAxis::TicksFixed) {
+            for (int i = 0; i < ticks; i++) {
+                qreal value = min + (i * (max - min) / (ticks - 1));
+                labels << formatLabel(formatSpec, array, value, precision, preStr, postStr);
+            }
+        } else {
+            qreal value = tickAnchor;
+            if (value > min)
+                value = value - int((value - min) / tickInterval) * tickInterval;
+            else
+                value = value + qCeil((min - value) / tickInterval) * tickInterval;
+
+            while (value <= max || qFuzzyCompare(value, max)) {
+                labels << formatLabel(formatSpec, array, value, precision, preStr, postStr);
+                value += tickInterval;
+            }
         }
     }
 
@@ -449,8 +549,6 @@ QStringList ChartAxisElement::createDateTimeLabels(qreal min, qreal max,int tick
     if (max <= min || ticks < 1)
         return labels;
 
-    int n = qMax(int(-qFloor(std::log10((max - min) / (ticks - 1)))), 0);
-    n++;
     for (int i = 0; i < ticks; i++) {
         qreal value = min + (i * (max - min) / (ticks - 1));
         labels << presenter()->locale().toString(QDateTime::fromMSecsSinceEpoch(value), format);
@@ -458,11 +556,39 @@ QStringList ChartAxisElement::createDateTimeLabels(qreal min, qreal max,int tick
     return labels;
 }
 
+
+bool ChartAxisElement::labelsEditable() const
+{
+    return m_labelsEditable;
+}
+
+void ChartAxisElement::setLabelsEditable(bool labelsEditable)
+{
+    if (axis()->type() == QAbstractAxis::AxisTypeValue
+            || axis()->type() == QAbstractAxis::AxisTypeDateTime) {
+        labelGroup()->setHandlesChildEvents(!labelsEditable);
+        const QList<QGraphicsItem *> childItems = labelGroup()->childItems();
+        for (auto item : childItems) {
+            switch (axis()->type()) {
+            case QtCharts::QAbstractAxis::AxisTypeValue:
+                static_cast<ValueAxisLabel *>(item)->setEditable(labelsEditable);
+                break;
+            case QtCharts::QAbstractAxis::AxisTypeDateTime:
+                static_cast<DateTimeAxisLabel *>(item)->setEditable(labelsEditable);
+                break;
+            default:
+                break;
+            }
+        }
+        m_labelsEditable = labelsEditable;
+    }
+}
+
 void ChartAxisElement::axisSelected()
 {
     emit clicked();
 }
 
-#include "moc_chartaxiselement_p.cpp"
-
 QT_CHARTS_END_NAMESPACE
+
+#include "moc_chartaxiselement_p.cpp"
